; :collapseFolds=1:maxLineLen=80:mode=autoitscript:tabSize=8:indentSize=8:folding=indent:
; #SCRIPT# =====================================================================
; Name.... ......: jEdit4AutoIt_UDF_Updater.au3
; Version .......: V2.1
; Description ...: Updates all files from jEdit4AutoIt, to the current UDFs:
;                : Xinsert: AutoItScript.insert.xml + ENVs
;                : Edit-Mode: au3.xml
;                : Include auto insert: autoitscript.xml
;                : Clipper: AutoItScript_UDFs.cliplibrary
; Author(s) .....: Thorsten Willert
; AutoIt Version : v3.3.0.0
; Modified ......: Wed Apr 08 16:25:02 CEST 2009 @642 /Internet Time/
;===============================================================================
#Region Includes
#include <Array.au3>
#include <Constants.au3>
#include <File.au3>
#include <GUIConstantsEx.au3>
#EndRegion Includes

Global $sTemplatePath = @ScriptDir & "\Templates\"
Global $sXInsertTemplate, $sXInsertFile, $sClipperTemplate, $sClipperFile

Global $sDirIncludes = RegRead("HKEY_LOCAL_MACHINE\SOFTWARE\AutoIt v3\AutoIt\", "InstallDir") & "\Include\"
Global $sDirUserIncludes = RegRead("HKEY_CURRENT_USER\Software\AutoIt v3\AutoIt", "Include")

If Not FileExists($sDirIncludes) Then
	MsgBox(64, "", "AutoIt includes not found in:" & @CRLF & $sDirIncludes)
	Exit
EndIf

GUICreate("jEdit4AutoIt_UDF_Updater", 340, 75)
Global $hStatus = GUICtrlCreateLabel("Updating:", 10, 10)
Global $hLabel = GUICtrlCreateLabel("", 60, 10, 270, 20)
Global $hFileLabel = GUICtrlCreateLabel("", 10, 30, 320, 20)
Global $hProgress1 = GUICtrlCreateProgress(10, 50, 320, 10)
Global $hProgress2 = GUICtrlCreateProgress(10, 60, 320, 10)
GUISetState(@SW_SHOW)

; Edit-Mode
Global $sEditModeTemplate = $sTemplatePath & "au3.xml"
Global $sEditModeFile = @UserProfileDir & "\.jEdit\modes\au3.xml"
If Not FileExists($sEditModeFile) Then $sEditModeFile = @ProgramFilesDir & "\jEdit\modes\au3.xml"

; Xinsert
If FileExists(@UserProfileDir & "\.jedit\xinsert\") Then
	$sXInsertTemplate = $sTemplatePath & "AutoItScript.insert.xml"
	$sXInsertFile = @UserProfileDir & "\.jedit\xinsert\AutoItScript.insert.xml"
EndIf

; Clipper
If FileExists(@UserProfileDir & "\.jedit\clipper\") Then
	$sClipperTemplate = $sTemplatePath & "AutoItScript_UDFs.cliplibrary"
	$sClipperFile = @UserProfileDir & "\.jedit\clipper\AutoItScript_UDFs.cliplibrary"
EndIf

GUICtrlSetData($hProgress1, 20)
_Label2($sDirIncludes)
Global $aIncludes = ReadIncludes($sDirIncludes)

; Includes auto insert
If FileExists(@UserProfileDir & "\.jedit\AutoItScript\") Then
	GUICtrlSetData($hLabel, "Include_Auto_Insert")
	GUICtrlSetData($hProgress1, 40)
	Global $sAutoInsertFile = @UserProfileDir & "\.jedit\AutoItScript\includes.xml"
	_Label2($sAutoInsertFile)
	AutoInsert_Update()
EndIf

If FileExists($sXInsertTemplate) Then
	GUICtrlSetData($hLabel, "XInsert ...")
	_Label2($sXInsertFile)
	GUICtrlSetData($hProgress1, 60)
	XInsert_Update()
EndIf
If FileExists($sEditModeTemplate) Then
	GUICtrlSetData($hLabel, "Edit Mode ...")
	_Label2($sEditModeFile)
	GUICtrlSetData($hProgress1, 80)
	EditMode_Update()
EndIf
If FileExists($sClipperTemplate) Then
	GUICtrlSetData($hLabel, "Clipper ...")
	_Label2($sClipperFile)
	GUICtrlSetData($hProgress1, 100)
	Clipper_Update()
EndIf

Sleep(1500)
GUICtrlSetData($hStatus, "Ready!")
GUICtrlSetData($hLabel, "")
_Label2("")
GUICtrlSetData($hProgress1, 100)
GUICtrlSetData($hProgress2, 100)

While 1 ;Message Loop
	If GUIGetMsg() = $GUI_EVENT_CLOSE Then Exit
	Sleep(50)
Wend ;==> Message Loop

Exit
;===============================================================================
Func AutoInsert_Update()
	ConsoleWrite("Updating includes.xml ..." & @CRLF)
	Local $sXML, $aFunc, $sFile, $i, $j

	$sXML = '<?xml version="1.0" ?>' & @CRLF
	$sXML &= '<!-- :collapseFolds=1:folding=indent: -->' & @CRLF
	$sXML &= '<include path="DEFAULT">' & @CRLF

	For $i = 1 To UBound($aIncludes) - 1
		$sFile = $aIncludes[$i][0]
		If $sFile = "" Then ContinueLoop
		$sXML &= @TAB & '<file name="' & $sFile & '">' & @CRLF

		$aFunc = StringSplit($aIncludes[$i][1], "|")
		For $j = 1 To $aFunc[0] - 1
			$sXML &= @TAB & @TAB & '<func>' & $aFunc[$j] & '</func>' & @CRLF
		Next
		$sXML &= @TAB & '</file>' & @CRLF

	Next
	$sXML &= '</include>' & @CRLF

	String2File($sAutoInsertFile, $sXML)
	Return
EndFunc   ;==>AutoInsert_Update
;===============================================================================
Func Clipper_Function_List()
	Local $aFunc
	Local $sFile, $i, $j

	Local $sXML = ""
	For $i = 1 To UBound($aIncludes) - 1
		GUICtrlSetData($hProgress2, 100 / UBound($aIncludes) * $i)
		$sFile = $aIncludes[$i][0]
		If $sFile = "" Then ContinueLoop

		$aFunc = StringSplit($aIncludes[$i][1], "|")
		For $j = 1 To $aFunc[0] - 1
			$sXML &= $aFunc[$j] & "=" & $aFunc[$j] & '(|)' & @CRLF
		Next

	Next
	GUICtrlSetData($hProgress2, 100)

	Return $sXML
EndFunc   ;==>Clipper_Function_List
;===============================================================================
Func Clipper_Update()
	ConsoleWrite("Updating Clipper ..." & @CRLF)
	Local $sData

	Local $sTemplate = File2String($sClipperTemplate)

	$sData = StringReplace($sTemplate, "UDF_INCLUDE 1uQBCpPGfQgIV-F971NY", Clipper_Function_List())

	String2File($sClipperFile, $sData)

	Return
EndFunc   ;==>Clipper_Update
;===============================================================================
Func EditMode_Function_List()
	Local $aFunc
	Local $sFile, $i, $j

	Local $sXML = "<!-- UDFs -->" & @CRLF
	For $i = 1 To UBound($aIncludes) - 1
		GUICtrlSetData($hProgress2, 100 / UBound($aIncludes) * $i)
		$sFile = $aIncludes[$i][0]
		If $sFile = "" Then ContinueLoop
		$sXML &= '<!-- ' & $sFile & ' -->' & @CRLF

		$aFunc = StringSplit($aIncludes[$i][1], "|")
		For $j = 1 To $aFunc[0] - 1
			$sXML &= StringFormat('<KEYWORD3>%s</KEYWORD3>', $aFunc[$j]) & @CRLF
		Next

	Next
	$sXML &= "<!-- END UDFs -->" & @CRLF
	GUICtrlSetData($hProgress2, 100)

	Return $sXML
EndFunc   ;==>EditMode_Function_List
;===============================================================================
Func EditMode_Update()
	ConsoleWrite("Updating Edit-Mode ..." & @CRLF)
	Local $sData

	Local $sTemplate = File2String($sEditModeTemplate)

	$sData = StringReplace($sTemplate, "<!-- UDF_INCLUDE 1uQBCpPGfQgIV-F971NY -->", EditMode_Function_List())

	String2File($sEditModeFile, $sData)

	Return
EndFunc   ;==>EditMode_Update
;===============================================================================
Func Xinsert_UDF_List($sMenu)
	Local $sFile, $sName, $i

	Local $sXML = Tab(4) & '<menu name="' & $sMenu & '">' & @CRLF
	For $i = 1 To UBound($aIncludes) - 1
		GUICtrlSetData($hProgress2, 100 / UBound($aIncludes) * $i)
		$sFile = $aIncludes[$i][0]
		If $sFile = "" Then ContinueLoop

		$sName = StripSuffix($sFile)
		$sXML &= Tab(5) & StringFormat('<item name="%s">#include &lt;%s&gt;</item>', $sName, $sFile) & @CRLF
	Next
	$sXML &= Tab(4) & '</menu>'
	GUICtrlSetData($hProgress2, 100)

	Return $sXML
EndFunc   ;==>Xinsert_UDF_List
;===============================================================================
Func Xinsert_UDF_Menu()
	Local $aFunc
	Local $sFile, $sName, $i, $j

	Local $sXML = Tab(2) & '<menu name="UDFs">' & @CRLF
	For $i = 1 To UBound($aIncludes) - 1
		GUICtrlSetData($hProgress2, 100 / UBound($aIncludes) * $i)
		$sFile = $aIncludes[$i][0]
		If $sFile = "" Then ContinueLoop

		$sName = StripSuffix($sFile)
		$sXML &= Tab(3) & StringFormat('<menu name="%s Management">', $sName) & @CRLF
		$sXML &= Tab(4) & StringFormat('<item name="Include %s">#include &lt;%s&gt;</item>', $sName, $sFile) & @CRLF

		$aFunc = StringSplit($aIncludes[$i][1], "|")
		_ArraySort($aFunc,0,1)
		For $j = 1 To $aFunc[0] - 1
			If $aFunc[$j] = "" Then ContinueLoop
			$sXML &= Tab(4) & StringFormat('<item name="%s">%s(|)</item>', $aFunc[$j], $aFunc[$j]) & @CRLF
		Next
		$sXML &= Tab(3) & "</menu>" & @CRLF
	Next
	$sXML &= Tab(2) & '</menu>'
	GUICtrlSetData($hProgress2, 100)

	Return $sXML
EndFunc   ;==>Xinsert_UDF_Menu
;===============================================================================
Func Xinsert_ENV_List()
	Local $i
	Local $sSet = Run(@ComSpec & " /c " & 'set', @SystemDir, Default, $STDOUT_CHILD)
	Local $line = ""
	While 1
		$line &= StdoutRead($sSet)
		If @error Then ExitLoop
	WEnd
	Local $aENV = StringRegExp($line, "(.*?)=.*?\n", 3)
	If Not @error And IsArray($aENV) Then
		Local $sXML = Tab(4) & '<menu name="Current ENVs">' & @CRLF
		For $i = 0 To UBound($aENV) - 1
			GUICtrlSetData($hProgress2, 100 / UBound($aIncludes) * $i)
			$sXML &= Tab(5) & StringFormat('<item name="%s">%s</item>', $aENV[$i], $aENV[$i]) & @CRLF
		Next
		$sXML &= Tab(4) & "</menu>" & @CRLF
	Else
		Return ""
	EndIf
	GUICtrlSetData($hProgress2, 100)

	Return $sXML
EndFunc   ;==>Xinsert_ENV_List
;===============================================================================
Func XInsert_Update()
	ConsoleWrite("Updating Xinsert ..." & @CRLF)
	Local $sUSER_Menu = "", $sData

	Local $sTemplate = File2String($sXInsertTemplate)

	GUICtrlSetData($hLabel, "XInsert UDF-menu ...")
	$sData = StringReplace($sTemplate, "<!-- UDF_INCLUDE 1uQBCpPGfQgIV-F971NY -->", Xinsert_UDF_Menu())
	$sData = StringReplace($sData, "<!-- UDF_USERINCLUDE 1uQBCpPGfQgIV-F971NY -->", $sUSER_Menu) ; not used
	GUICtrlSetData($hLabel, "XInsert UDF-list ...")
	$sData = StringReplace($sData, "<!-- UDF_INCLUDE_LIST 1uQBCpPGfQgIV-F971NY -->", Xinsert_UDF_List("UDFs"))
	GUICtrlSetData($hLabel, "XInsert ENV-list ...")
	$sData = StringReplace($sData, "<!-- ENV 1uQBCpPGfQgIV-F971NY -->", Xinsert_ENV_List())

	String2File($sXInsertFile, $sData)

	Return
EndFunc   ;==>XInsert_Update
;===============================================================================
Func ReadFunctions($sFile, $bInternal)
	Local $aFile, $sRet
	Local $sRegEx = "^[\s]*[Ff]unc[\s]+([\w]+)"
	Local $aRegEx

	_FileReadToArray($sFile, $aFile)

	For $i = 1 To $aFile[0]
		$aRegEx = StringRegExp($aFile[$i], $sRegEx, 3)
		If IsArray($aRegEx) Then
			If Not $bInternal And StringLeft($aRegEx[0], 2) = "__" Then ContinueLoop
			$sRet &= $aRegEx[0] & "|"
		EndIf
	Next

	Return $sRet
EndFunc   ;==>ReadFunctions
;===============================================================================
Func ReadConstants($sFile)
	Local $aFile, $sRet
	Local $sRegEx = "^[\s]*[Gg]lobal[\s]*[Cc]onst[\s]*(\$[\w]+)(.*?)$"
	Local $aRegEx

	_FileReadToArray($sFile, $aFile)

	For $i = 1 To $aFile[0]
		$aRegEx = StringRegExp($aFile[$i], $sRegEx, 3)
		If IsArray($aRegEx) Then $sRet &= $aRegEx[0] & "|"
	Next

	Return $sRet
EndFunc   ;==>ReadConstants
;===============================================================================
Func ReadIncludes($sDir, $bInternal = False)
	Local $aFiles = _FileListToArray($sDirIncludes, "*.au3", 1)
	Local $aRet[$aFiles[0] + 1][2]

	For $i = 1 To $aFiles[0]
		If Not StringInStr($aFiles[$i], "constants") And Not StringInStr($aFiles[$i], "dll") And Not StringInStr($aFiles[$i], "~") Then
			GUICtrlSetData($hLabel, "Analyzing: " & $aFiles[$i] & "...")
			GUICtrlSetData($hProgress2, 100 / $aFiles[0] * $i + 1)
			$aRet[$i][0] = $aFiles[$i]
			$aRet[$i][1] = ReadFunctions($sDir & $aFiles[$i], $bInternal)
		EndIf
	Next

	Return $aRet
EndFunc   ;==>ReadIncludes
;===============================================================================
Func File2String($sFile)
	Local $hFile = FileOpen($sFile, 0)
	Local $sTemp = FileRead($hFile)
	FileClose($hFile)
	Return $sTemp
EndFunc   ;==>File2String
;===============================================================================
Func String2File($sFile, $sString)
	Local $hFile = FileOpen($sFile, 2)
	FileWrite($hFile, $sString)
	FileClose($hFile)
	Return
EndFunc   ;==>String2File
;===============================================================================
Func StripSuffix($sFile)
	Return StringMid($sFile, 1, StringInStr($sFile, ".") - 1)
EndFunc   ;==>StripSuffix
;===============================================================================
Func Tab($n)
	Local $tmp = ''
	For $i = 1 To $n
		$tmp &= @TAB
	Next
	Return $tmp
EndFunc   ;==>Tab
;===============================================================================
Func _Label2($sString)
	If StringLen($sString) > 70 Then $sString = "..." & StringRight($sString, 70)
	GUICtrlSetData($hFileLabel, $sString)
EndFunc   ;==>_Label2